<?php

declare(strict_types=1);

namespace Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Action;

use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\ApiInterface as SogenactifApi;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Generator\TransactionInterface;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\SealEncoderInterface;
use GuzzleHttp\Client;
use Http\Client\Exception\RequestException;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Exception\UnsupportedApiException;
use Payum\Core\Reply\HttpPostRedirect;
use Payum\Core\Request\Capture as PayumRequestCapture;
use Psr\Http\Message\ResponseInterface;
use Sylius\Bundle\PayumBundle\Model\PaymentSecurityTokenInterface;
use Sylius\Component\Core\Model\PaymentInterface as SyliusPaymentInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Routing\RouterInterface;

final class Capture implements ActionInterface, ApiAwareInterface
{
    public const CURRENCY_CODE_EURO = '978';

    public const INTERFACE_VERSION = 'IR_WS_2.3';

    public const ORDER_CHANNEL = 'INTERNET';

    private Client $client;

    private SogenactifApi $api;

    private SealEncoderInterface $sealEncoder;

    private RequestStack $requestStack;

    private TransactionInterface $transaction;

    private RouterInterface $router;

    private string $fqdnUrlInitializeTransaction;

    private bool $shouldGenerateTransactionReference;

    public function __construct(
        Client               $client,
        SealEncoderInterface $sealEncoder,
        RequestStack         $requestStack,
        TransactionInterface $transaction,
        RouterInterface      $router,
        string               $fqdnUrlInitializeTransaction,
        bool                 $shouldGenerateTransactionReference
    ) {
        $this->client = $client;
        $this->sealEncoder = $sealEncoder;
        $this->requestStack = $requestStack;
        $this->transaction = $transaction;
        $this->router = $router;
        $this->fqdnUrlInitializeTransaction = $fqdnUrlInitializeTransaction;
        $this->shouldGenerateTransactionReference = $shouldGenerateTransactionReference;
    }

    /**
     * @param PayumRequestCapture|mixed $request
     */
    public function execute($request): void
    {
        RequestNotSupportedException::assertSupports($this, $request);

        /** @var SyliusPaymentInterface $payment */
        $payment = $request->getModel();

        /** @var PaymentSecurityTokenInterface $paymentSecurityToken */
        $paymentSecurityToken = $request->getToken();

        /** @var Request $currentRequest */
        $currentRequest = $this->requestStack->getCurrentRequest();

        /** @var string|null $dataRequest */
        $dataRequest = $currentRequest->request->get('Data');

        // @TODO Cur for one responsability
        if ($dataRequest === null) {
            $automaticResponseUrl = $paymentSecurityToken->getTargetUrl();
            $normalReturnUrl = $this->router->generate('dedi_sogenactif_plugin_shop_order_confirm', [], UrlGeneratorInterface::ABSOLUTE_URL);
            $order = $payment->getOrder();

            if (null === $order) {
                throw new \RuntimeException(sprintf('Could not resolve order for payment %s', $payment->getId()));
            }

            $channel = $order->getChannel();

            if (null === $channel) {
                throw new \RuntimeException(sprintf('Could not resolve channel for order %s', $order->getId()));
            }

            // Data needed for create seal
            $infos = [
                'amount' => (string) $order->getTotal(),
                'automaticResponseUrl' => $automaticResponseUrl,
                'currencyCode' => $this::CURRENCY_CODE_EURO, // @TODO check for FR
                'interfaceVersion' => $this::INTERFACE_VERSION,
                'merchantId' => $this->api->getMerchantId(),
                'normalReturnUrl' => $normalReturnUrl,
                'orderChannel' => $this::ORDER_CHANNEL,
                'orderId' => $order->getId()
            ];

            if ($this->shouldGenerateTransactionReference) {
                $infos['transactionReference'] = $this->transaction->generate($channel->getCode());
            }

            $infos['seal'] = $this->sealEncoder->createSeal($infos, $this->api->getSecretKey());
            $infos['keyVersion'] = $this->api->getKeyVersion();

            try {
                /** @var ResponseInterface $response */
                $response = $this
                    ->client
                    ->post($this->fqdnUrlInitializeTransaction, [
                        'json' => $infos,
                    ])
                ;

                $payment->setDetails(['status' => 'processing']);

                $response = json_decode($response->getBody()->getContents(), true);

                // @TODO more check needed
                throw new HttpPostRedirect(
                    $response['redirectionUrl'], [
                        'redirectionVersion' => $response['redirectionVersion'],
                        'redirectionData' => $response['redirectionData'],
                    ]
                );
            } catch (RequestException $exception) {
                $errorMessage = $exception->getMessage();
                $payment->setDetails(['error_message' => $errorMessage]);
            }
        } else {
            $datas = explode('|', $dataRequest);

            $normalizeData = [];
            $details = [];

            foreach ($datas as $data) {
                if (strpos($data, '=')) {
                    $explodedData = explode('=', $data);
                    if (array_key_exists(0, $explodedData) && array_key_exists(1, $explodedData)) {
                        $normalizeData[$explodedData[0]] = $explodedData[1];
                    } else {
                        $details['error_data_from_return'][] = 'Key value pair for normalized data is null for data receive ' . $data;
                    }
                } else {
                    $details['error_data_from_return'][] = 'No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is ' . $data;
                }
            }

            $details['request_data_from_payback'] = $dataRequest;
            $details['all_normalized_infos'] = $normalizeData;

            $payment->setDetails($details);
        }
    }

    public function supports($request): bool
    {
        return
            $request instanceof PayumRequestCapture &&
            $request->getModel() instanceof SyliusPaymentInterface
            ;
    }

    public function setApi($api): void
    {
        if (!$api instanceof SogenactifApi) {
            throw new UnsupportedApiException('Not supported. Expected an instance of ' . SogenactifApi::class);
        }

        $this->api = $api;
    }
}
